@@ -21,6 +21,7 @@ import typing
 import functools
 import collections
 import inspect
+import sys
 
 import hikari
 from hikari.events import message
@@ -29,6 +30,7 @@ from lightbulb import commands
 from lightbulb import context
 from lightbulb import errors
 from lightbulb import stringview
+from lightbulb import plugins
 
 if typing.TYPE_CHECKING:
     from hikari.models import messages
@@ -92,13 +94,13 @@ class BotWithHandler(hikari.Bot):
 
         self.ignore_bots: bool = ignore_bots
         self.owner_ids: typing.Iterable[int] = owner_ids
+
+        self.extensions = []
+        self.plugins: typing.MutableMapping[str, plugins.Plugin] = {}
         self.commands: typing.MutableMapping[
             str, typing.Union[commands.Command, commands.Group]
         ] = {}
 
-    async def _default_command_error(self, event: errors.CommandErrorEvent) -> None:
-        raise event.error
-
     async def fetch_owner_ids(self) -> None:
         """
         Fetches the IDs of the bot's owner(s) from the API and stores them in
@@ -115,16 +117,12 @@ class BotWithHandler(hikari.Bot):
         if application.team is not None:
             self.owner_ids.extend([member_id for member_id in application.team.members])
 
-    def command(
-        self, *, allow_extra_arguments: bool = True, name: typing.Optional[str] = None
-    ) -> typing.Callable:
+    def command(self, **kwargs) -> typing.Callable:
         """
         A decorator that registers a callable as a command for the handler.
 
-        Args:
-            allow_extra_arguments (:obj:`bool`): Whether or not the handler should raise an error if the command is run
-                with more arguments than it requires. Defaults to True.
-            name (:obj:`str`): Optional name of the command. Defaults to the name of the function if not specified.
+        Keyword Args:
+            **kwargs: See :obj:`.commands.command` for valid kwargs.
 
         Example:
 
@@ -143,27 +141,25 @@ class BotWithHandler(hikari.Bot):
 
         def decorate(func: typing.Callable):
             nonlocal registered_commands
-            nonlocal allow_extra_arguments
-            nonlocal name
-            name = func.__name__ if name is None else name
-            if not registered_commands.get(name):
-                registered_commands[name] = commands.Command(
-                    func, allow_extra_arguments, name
-                )
-                return registered_commands[name]
+            name = kwargs.get("name", func.__name__)
+            registered_commands[name] = commands.Command(
+                func,
+                name,
+                kwargs.get("allow_extra_arguments", True),
+                kwargs.get("aliases", []),
+            )
+            for alias in kwargs.get("aliases", []):
+                registered_commands[alias] = registered_commands[name]
+            return registered_commands[name]
 
         return decorate
 
-    def group(
-        self, *, allow_extra_arguments: bool = True, name: typing.Optional[str] = None
-    ) -> typing.Callable:
+    def group(self, **kwargs) -> typing.Callable:
         """
         A decorator that registers a callable as a command group for the handler.
 
-        Args:
-            allow_extra_arguments (:obj:`bool`): Whether or not the handler should raise an error if the command is run
-                with more arguments than it requires. Defaults to True.
-            name (:obj:`str`): Optional name of the command. Defaults to the name of the function if not specified.
+        Keyword Args:
+            **kwargs: See :obj:`.commands.group` for valid kwargs.
 
         Example:
 
@@ -182,35 +178,31 @@ class BotWithHandler(hikari.Bot):
 
         def decorate(func: typing.Callable):
             nonlocal registered_commands
-            nonlocal allow_extra_arguments
-            nonlocal name
-            name = func.__name__ if name is None else name
-            if not registered_commands.get(name):
-                registered_commands[name] = commands.Group(
-                    func, allow_extra_arguments, name
-                )
-                return registered_commands.get(name)
+            name = kwargs.get("name", func.__name__)
+            registered_commands[name] = commands.Group(
+                func,
+                name,
+                kwargs.get("allow_extra_arguments", True),
+                kwargs.get("aliases", []),
+            )
+            for alias in kwargs.get("aliases", []):
+                registered_commands[alias] = registered_commands[name]
+            return registered_commands[name]
 
         return decorate
 
-    def add_command(
-        self,
-        func: typing.Callable,
-        *,
-        allow_extra_arguments=True,
-        name: typing.Optional[str] = None,
-    ) -> None:
+    def add_command(self, func: typing.Callable, **kwargs) -> commands.Command:
         """
         Adds a command to the bot. Similar to the ``command`` decorator.
 
         Args:
             func (:obj:`typing.Callable`): The function to add as a command.
-            allow_extra_arguments (:obj:`bool`): Whether or not the handler should raise an error if the command is run
-                with more arguments than it requires. Defaults to True.
-            name (:obj:`str`): Optional name of the command. Defaults to the name of the function if not specified.
+
+        Keyword Args:
+            **kwargs: See :obj:`.commands.command` for valid kwargs.
 
         Returns:
-            ``None``
+            :obj:`commands.Command` that was added to the bot.
 
         Example:
 
@@ -226,40 +218,67 @@ class BotWithHandler(hikari.Bot):
         See Also:
             :meth:`.command_handler.BotWithHandler.command`
         """
-        name = func.__name__ if name is None else name
-        if not self.commands.get(name):
-            self.commands[name] = commands.Command(func, allow_extra_arguments, name)
+        if not isinstance(func, commands.Command):
+            name = kwargs.get("name", func.__name__)
+            self.commands[name] = commands.Command(
+                func,
+                name,
+                kwargs.get("allow_extra_arguments", True),
+                kwargs.get("aliases", []),
+            )
+            for alias in kwargs.get("aliases", []):
+                self.commands[alias] = self.commands[name]
+        else:
+            name = func.name
+            self.commands[name] = func
+            for alias in func._aliases:
+                self.commands[alias] = func
+        return self.commands[name]
 
-    def add_group(
-        self,
-        func: typing.Callable,
-        *,
-        allow_extra_arguments=True,
-        name: typing.Optional[str] = None,
-    ) -> None:
+    def add_group(self, func: typing.Callable, **kwargs) -> commands.Group:
         """
         Adds a command group to the bot. Similar to the ``group`` decorator.
 
         Args:
             func (:obj:`typing.Callable`): The function to add as a command group.
-            allow_extra_arguments (:obj:`bool`): Whether or not the handler should raise an error if the command is run
-                with more arguments than it requires. Defaults to True.
-            name (:obj:`str`): Optional name of the command group. Defaults to the name of the function if not specified.
+
+        Keyword Args:
+            **kwargs: See :obj:`.commands.group` for valid kwargs.
 
         Returns:
-            ``None``
+            :obj:`commands.Group` that was added to the bot.
 
         See Also:
             :meth:`.command_handler.BotWithHandler.group`
             :meth:`.command_handler.BotWithHandler.add_command`
         """
-        name = func.__name__ if name is None else name
-        if not self.commands.get(name):
-            self.commands[name] = commands.Group(func, allow_extra_arguments, name)
+        name = kwargs.get("name", func.__name__)
+        self.commands[name] = commands.Group(
+            func,
+            name,
+            kwargs.get("allow_extra_arguments", True),
+            kwargs.get("aliases", []),
+        )
+        for alias in kwargs.get("aliases", []):
+            self.commands[alias] = self.commands[name]
+        return self.commands[name]
+
+    def add_plugin(self, plugin: plugins.Plugin) -> None:
+        """
+        Add a :obj:`.plugins.Plugin` to the bot including all of the plugin commands.
+
+        Args:
+            plugin (:obj:`.plugins.Plugin`): Plugin to add to the bot.
+
+        Returns:
+            ``None``
+        """
+        self.plugins[plugin.name] = plugin
+        self.commands.update(plugin.commands)
 
     def get_command(self, name: str) -> typing.Optional[commands.Command]:
         """
-        Get a command object from it's registered name.
+        Get a command object from its registered name.
 
         Args:
             name (:obj:`str`): The name of the command to get the object for.
@@ -269,6 +288,18 @@ class BotWithHandler(hikari.Bot):
         """
         return self.commands.get(name)
 
+    def get_plugin(self, name: str) -> typing.Optional[plugins.Plugin]:
+        """
+        Get a plugin object from its registered name.
+
+        Args:
+             name (:obj:`str`): The name of the plugin to get the object for.
+
+        Returns:
+            Optional[ :obj:`.commands.Command` ] plugin object registered to that name.
+        """
+        return self.plugins.get(name)
+
     def remove_command(self, name: str) -> typing.Optional[str]:
         """
         Remove a command from the bot and return its name or ``None`` if no command was removed.
@@ -282,6 +313,124 @@ class BotWithHandler(hikari.Bot):
         command = self.commands.pop(name)
         return command.name if command is not None else None
 
+    def remove_plugin(self, name: str) -> typing.Optional[str]:
+        """
+        Remove a plugin from the bot and return its name or ``None`` if no plugin was removed.
+
+        Args:
+            name (:obj:`str`): The name of the plugin to remove.
+
+        Returns:
+            Optional[ :obj:`str` ] name of the plugin that was removed.
+        """
+        plugin = self.plugins.pop(name)
+        if plugin is not None:
+            for k in plugin.commands.keys():
+                self.commands.pop(k)
+        return plugin.name if plugin is not None else None
+
+    def load_extension(self, extension: str) -> None:
+        """
+        Load an external extension into the bot. Extension name follows the format ``<directory>.<filename>``
+        The extension **must** contain a function ``load`` which takes a single argument which will be the
+        bot instance you are loading the extension into.
+
+        Args:
+            extension (:obj:`str`): The name of the extension to load.
+
+        Returns:
+            ``None``
+
+        Raises:
+            ExtensionAlreadyLoaded: If the extension has already been loaded.
+            ExtensionMissingLoad: If the extension to be loaded does not contain a ``load`` function.
+
+        Example:
+            This method is useful when wanting to split your bot up into multiple files.
+            An example extension is seen below  - ``example.py`` and would be loaded by calling
+            the following: ``bot.load_extension("example")``
+
+            .. code-block:: python
+
+                from lightbulb import plugins
+
+                class MyPlugin(plugins.Plugin):
+                    ...
+
+                def load(bot):
+                    bot.add_plugin(MyPlugin())
+
+        """
+        if extension in self.extensions:
+            raise errors.ExtensionAlreadyLoaded(f"{extension} is already loaded.")
+
+        paths = extension.split(".")
+        module = __import__(extension)
+
+        submodule = None
+        for path in paths[1:]:
+            submodule = getattr(module if submodule is None else submodule, path)
+
+        load_location = module if submodule is None else submodule
+
+        if not hasattr(load_location, "load"):
+            raise errors.ExtensionMissingLoad(f"{extension} is missing a load function")
+        else:
+            load_location.load(self)
+            self.extensions.append(extension)
+
+    def unload_extension(self, extension: str) -> None:
+        """
+        Unload an external extension from the bot. This method relies on a function, ``unload``
+        existing in the extension which the bot will use to remove all commands and/or plugins
+        from the bot.
+
+        Args:
+            extension (:obj:`str`): The name of the extension to unload.
+
+        Returns:
+            ``None``
+
+        Raises:
+            ExtensionNotLoaded: If the extension has not been loaded.
+            ExtensionMissingUnload: If the extension does not contain an ``unload`` function.
+
+        Example:
+
+            .. code-block:: python
+
+                from lightbulb import plugins
+
+                class MyPlugin(plugins.Plugin):
+                    ...
+
+                def load(bot):
+                    bot.add_plugin(MyPlugin())
+
+                def unload(bot):
+                    bot.remove_plugin("MyPlugin")
+        """
+        if extension not in self.extensions:
+            raise errors.ExtensionNotLoaded(f"{extension} is not loaded.")
+
+        paths = extension.split(".")
+        module = __import__(extension)
+
+        submodule = None
+        for path in paths[1:]:
+            submodule = getattr(module if submodule is None else submodule, path)
+
+        unload_location = module if submodule is None else submodule
+
+        if not hasattr(unload_location, "unload"):
+            raise errors.ExtensionMissingUnload(
+                f"{extension} is missing an unload function"
+            )
+        else:
+            unload_location.unload(self)
+            self.extensions.remove(extension)
+            del sys.modules[extension]
+
     def resolve_arguments(
         self, message: messages.Message, prefix: str
     ) -> typing.List[str]:
@@ -306,33 +455,27 @@ class BotWithHandler(hikari.Bot):
     async def _evaluate_checks(
         self, command: commands.Command, context: context.Context
     ):
-        for check in command.checks:
+        failed_checks = []
+
+        for check in command._checks:
             try:
-                check_pass = await check(context)
-                if not check_pass:
-                    raise errors.CheckFailure(
-                        f"Check {check.__name__} failed for command {context.invoked_with}"
+                if not await check(context):
+                    failed_checks.append(
+                        errors.CheckFailure(
+                            f"Check {check.__name__} failed for command {context.invoked_with}"
+                        )
                     )
-            except errors.OnlyInGuild as e:
-                self.event_dispatcher.dispatch(
-                    errors.CommandErrorEvent(e, context.message)
-                )
-                return False
-            except errors.OnlyInDM as e:
-                self.event_dispatcher.dispatch(
-                    errors.CommandErrorEvent(e, context.message)
-                )
-                return False
-            except errors.NotOwner as e:
-                self.event_dispatcher.dispatch(
-                    errors.CommandErrorEvent(e, context.message)
-                )
-                return False
-            except errors.CheckFailure as e:
-                self.event_dispatcher.dispatch(
-                    errors.CommandErrorEvent(e, context.message)
-                )
-                return False
+            except Exception as ex:
+                error = errors.CheckFailure(str(ex))
+                error.__cause__ = ex
+                failed_checks.append(ex)
+
+        if len(failed_checks) > 1:
+            raise errors.CheckFailure(
+                "Multiple checks failed: " + ", ".join(str(ex) for ex in failed_checks)
+            )
+        elif failed_checks:
+            raise failed_checks[0]
         return True
 
     async def _invoke_command(
@@ -341,33 +484,29 @@ class BotWithHandler(hikari.Bot):
         context: context.Context,
         args: typing.List[str],
     ) -> None:
-        if not await self._evaluate_checks(command, context):
-            return
+        try:
+            if not await self._evaluate_checks(command, context):
+                return
 
-        if not command._has_max_args and len(args) >= command._min_args:
-            await command(context, *args)
+            if not command._has_max_args and len(args) >= command._min_args:
+                await command.invoke(context, *args)
 
-        elif len(args) < command._max_args:
-            self.event_dispatcher.dispatch(
-                errors.CommandErrorEvent(
-                    errors.NotEnoughArguments(context.invoked_with), context.message
-                )
-            )
-            return
+            elif len(args) < command._min_args:
+                raise errors.NotEnoughArguments(context.invoked_with)
 
-        elif len(args) > command._max_args and not command.allow_extra_arguments:
-            self.event_dispatcher.dispatch(
-                errors.CommandErrorEvent(
-                    errors.TooManyArguments(context.invoked_with), context.message
-                )
-            )
-            return
+            elif len(args) > command._max_args and not command._allow_extra_arguments:
+                raise errors.TooManyArguments(context.invoked_with)
 
-        elif command._max_args == 0:
-            await command(context)
+            elif command._max_args == 0:
+                await command.invoke(context)
 
-        else:
-            await command(context, *args[: command._max_args + 1])
+            else:
+                await command.invoke(context, *args[: command._max_args + 1])
+        except errors.CommandError as ex:
+            if self.get_listeners(errors.CommandErrorEvent, polymorphic=True):
+                await self.dispatch(errors.CommandErrorEvent(ex, context.message))
+            else:
+                raise
 
     async def handle(self, event: message.MessageCreateEvent) -> None:
         """
@@ -407,27 +546,19 @@ class BotWithHandler(hikari.Bot):
 
         invoked_with = args[0]
         if invoked_with not in self.commands:
-            self.event_dispatcher.dispatch(
-                errors.CommandErrorEvent(
-                    errors.CommandNotFound(invoked_with), event.message
-                )
-            )
-            return
+            raise errors.CommandNotFound(invoked_with)
 
         invoked_command = self.commands[invoked_with]
-        if isinstance(invoked_command, commands.Command):
-            new_args = args[1:]
+
         if isinstance(invoked_command, commands.Group):
-            invoked_command, new_args = invoked_command.resolve_subcommand(args)
+            try:
+                invoked_command, new_args = invoked_command._resolve_subcommand(args)
+            except AttributeError:
+                new_args = args[1:]
+        else:
+            new_args = args[1:]
 
         command_context = context.Context(
             self, event.message, prefix, invoked_with, invoked_command
         )
         await self._invoke_command(invoked_command, command_context, new_args)
-
-    def run(self) -> None:
-        if errors.CommandErrorEvent not in self.event_dispatcher._listeners:
-            self.event_dispatcher.subscribe(
-                errors.CommandErrorEvent, self._default_command_error
-            )
-        super().run()
