@@ -44,11 +44,42 @@ class CommandErrorEvent(base.Event):
 
     def __init__(self, error, message) -> None:
         self.error = error
+        self.traceback = error.__traceback__
         self.message = message
 
 
+class ExtensionError(Exception):
+    """Base exception for errors incurred during the loading and unloading of extensions."""
+
+    pass
+
+
+class ExtensionAlreadyLoaded(ExtensionError):
+    """Exception raised when an extension already loaded is attempted to be loaded."""
+
+    pass
+
+
+class ExtensionNotLoaded(ExtensionError):
+    """Exception raised when an extension not already loaded is attempted to be unloaded."""
+
+    pass
+
+
+class ExtensionMissingLoad(ExtensionError):
+    """Exception raised when an extension is attempted to be loaded but does not contain a load function"""
+
+    pass
+
+
+class ExtensionMissingUnload(ExtensionError):
+    """Exception raised when an extension is attempted to be unloaded but does not contain an unload function"""
+
+    pass
+
+
 class CommandError(Exception):
-    """Base exception for the command handler."""
+    """Base exception for errors incurred during handling od commands."""
 
     pass
 
@@ -106,13 +137,9 @@ class CheckFailure(CommandError):
     """
     Base error that is raised when a check fails for a command. Anything raised by a check
     should inherit from this class.
-
-    Args:
-        context (:obj:`.context.Context`): The context that caused the check to fail.
     """
 
-    def __init__(self, context: context.Context) -> None:
-        self.context = context
+    pass
 
 
 class OnlyInGuild(CheckFailure):
